<?php
if (!defined('__TYPECHO_ROOT_DIR__')) exit;

/**
 * 聚合图床外链插件
 *
 * @package Superbed
 * @author 聚合图床
 * @version 1.0.0
 * @link https://www.superbed.cn
 */
class Superbed_Plugin implements Typecho_Plugin_Interface
{
    const UPLOAD_DIR  = '/usr/uploads'; //上传文件目录路径
    const PLUGIN_NAME = 'Superbed'; //插件名称

    public static function activate()
    {
        Typecho_Plugin::factory('Widget_Upload')->uploadHandle     = array(__CLASS__, 'uploadHandle');
        Typecho_Plugin::factory('Widget_Upload')->modifyHandle     = array(__CLASS__, 'modifyHandle');
        Typecho_Plugin::factory('Widget_Upload')->deleteHandle     = array(__CLASS__, 'deleteHandle');
        Typecho_Plugin::factory('Widget_Upload')->attachmentHandle = array(__CLASS__, 'attachmentHandle');
    }

    public static function deactivate()
    {

    }

    public static function config(Typecho_Widget_Helper_Form $form)
    {
        $desc = new Typecho_Widget_Helper_Form_Element_Text('desc', NULL, '', '插件介绍：', '聚合图床外链插件');
        $form->addInput($desc);

        $token = new Typecho_Widget_Helper_Form_Element_Text('token', NULL, '', 'Token：', '请输入您的token');
        $form->addInput($token);

        $categories = new Typecho_Widget_Helper_Form_Element_Text('categories', NULL, '', '相册：', '请填写相册名，可留空');
        $form->addInput($categories);

        echo '<script>window.onload = function(){document.getElementsByName("desc")[0].type = "hidden";}</script>';
    }

    public static function personalConfig(Typecho_Widget_Helper_Form $form)
    {
    }

    public static function uploadHandle($file)
    {
        if (empty($file['name'])) {

            return false;
        }

        //获取扩展名
        $ext = self::_getSafeName($file['name']);
        //判定是否是允许的文件类型
        if (!Widget_Upload::checkFileType($ext) || Typecho_Common::isAppEngine()) {

            return false;
        }

        // 判断是否是图片
        if (self::_isImage($ext)) {

            return self::_uploadImg($file, $ext);
        }

        return self::_uploadOtherFile($file, $ext);
    }

    public static function deleteHandle(array $content)
    {
        $ext = self::_getSafeName($content['title']);
        if (self::_isImage($ext)) {

            return self::_deleteImg($content);
        }

        return unlink($content['attachment']->path);
    }

    public static function modifyHandle($content, $file)
    {
        if (empty($file['name'])) {

            return false;
        }

        $ext = self::_getSafeName($file['name']);
        if ($content['attachment']->type != $ext || Typecho_Common::isAppEngine()) {

            return false;
        }

        if (!self::_getUploadFile($file)) {

            return false;
        }

        if (self::_isImage($ext)) {
            self::_deleteImg($content);

            return self::_uploadImg($file, $ext);
        }

        return self::_uploadOtherFile($file, $ext);
    }

    public static function attachmentHandle(array $content)
    {
        $arr = unserialize($content['text']);
        $ext = self::_getSafeName($content['title']);
        if (self::_isImage($ext)) {

            return $content['attachment']->path ?? '';
        }

        $ret = explode(self::UPLOAD_DIR, $arr['path']);
        return Typecho_Common::url(self::UPLOAD_DIR . $ret[1], Helper::options()->siteUrl);
    }

    private static function _getUploadDir($ext = '')
    {
        if (self::_isImage($ext)) {
            $url = parse_url(Helper::options()->siteUrl);
            $DIR = str_replace('.', '_', $url['host']);
            return '/' . $DIR . self::UPLOAD_DIR;
        } elseif (defined('__TYPECHO_UPLOAD_DIR__')) {
            return __TYPECHO_UPLOAD_DIR__;
        } else {
            $path = Typecho_Common::url(self::UPLOAD_DIR, __TYPECHO_ROOT_DIR__);
            return $path;
        }
    }

    private static function _getUploadFile($file)
    {
        return $file['tmp_name'] ?? ($file['bytes'] ?? ($file['bits'] ?? ''));
    }

    private static function _getSafeName(&$name)
    {
        $name = str_replace(array('"', '<', '>'), '', $name);
        $name = str_replace('\\', '/', $name);
        $name = false === strpos($name, '/') ? ('a' . $name) : str_replace('/', '/a', $name);
        $info = pathinfo($name);
        $name = substr($info['basename'], 1);

        return isset($info['extension']) ? strtolower($info['extension']) : '';
    }

    private static function _makeUploadDir($path)
    {
        $path    = preg_replace("/\\\+/", '/', $path);
        $current = rtrim($path, '/');
        $last    = $current;

        while (!is_dir($current) && false !== strpos($path, '/')) {
            $last    = $current;
            $current = dirname($current);
        }

        if ($last == $current) {
            return true;
        }

        if (!@mkdir($last)) {
            return false;
        }

        $stat  = @stat($last);
        $perms = $stat['mode'] & 0007777;
        @chmod($last, $perms);

        return self::_makeUploadDir($path);
    }

    private static function _isImage($ext)
    {
        $img_ext_arr = array('gif', 'jpg', 'jpeg', 'png', 'webp', 'bmp');
        return in_array($ext, $img_ext_arr);
    }

    private static function _uploadOtherFile($file, $ext)
    {
        $dir = self::_getUploadDir($ext) . '/' . date('Y') . '/' . date('m');
        if (!self::_makeUploadDir($dir)) {

            return false;
        }

        $path = sprintf('%s/%u.%s', $dir, crc32(uniqid()), $ext);
        if (!isset($file['tmp_name']) || !@move_uploaded_file($file['tmp_name'], $path)) {

            return false;
        }

        return [
            'name' => $file['name'],
            'path' => $path,
            'size' => $file['size'] ?? filesize($path),
            'type' => $ext,
            'mime' => @Typecho_Common::mimeContentType($path)
        ];
    }

    private static function _uploadImg($file, $ext)
    {
        $options = Helper::options()->plugin(self::PLUGIN_NAME);
        $api     = 'https://api.superbed.cn/upload';
        $token   = $options->token;
        $tmp     = self::_getUploadFile($file);
        if (empty($tmp)) {
            return false;
        }

        $img = $tmp . '.' . $ext;
        if (!rename($tmp, $img)) {
            return false;
        }

        $post = array(
            'token' => $options->token,
            'categories' => $options->categories,
            'file' => new CURLFile($img),
        );
        $res = self::_curlPost($api, $post);
        unlink($img);

        if (!$res) {
            return false;
        }

        $json = json_decode($res, true);
        if ($json['err'] != 0) {
            return false;
        }

        return [
            'img_id' => $json['id'],
            'path'   => $json['url'],
            'name'   => $file['name'],
            'size'   => $json['size'],
            'mime'   => $json['mime'],
            'type'   => $ext,
        ];
    }

    private static function _deleteImg(array $content)
    {
        $options = Helper::options()->plugin(self::PLUGIN_NAME);
        if (empty($token)) {
            return true;
        }

        $id = $content['attachment']->img_id;
        if (empty($id)) {
            return false;
        }

        $api     = 'https://api.superbed.cn/info/' . $id;
        $post = array('token' => $options->token, 'action' => 'delete');
        $res  = self::_curlPost($api, $post);
        $json = json_decode($res, true);
        if (!is_array($json)) {
            return false;
        }

        return $json['err'] == 0;
    }

    private static function _curlPost($api, $post)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $api . '?token=' . $token);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, false);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_USERAGENT, 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/85.0.4183.102 Safari/537.36');
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
        $res = curl_exec($ch);
        curl_close($ch);

        return $res;
    }
}
